@@ -1,6 +1,6 @@
 /* statement.c - the statement type
  *
- * Copyright (C) 2005-2007 Gerhard Hring <gh@ghaering.de>
+ * Copyright (C) 2005-2015 Gerhard Hring <gh@ghaering.de>
  *
  * This file is part of pysqlite.
  *
@@ -167,12 +167,14 @@ int pysqlite_statement_bind_parameter(pysqlite_Statement* self, int pos, PyObjec
             break;
         case TYPE_STRING:
             string = PyString_AS_STRING(parameter);
-            rc = sqlite3_bind_text(self->st, pos, string, -1, SQLITE_TRANSIENT);
+            buflen = PyString_Size(parameter);
+            rc = sqlite3_bind_text(self->st, pos, string, buflen, SQLITE_TRANSIENT);
             break;
         case TYPE_UNICODE:
             stringval = PyUnicode_AsUTF8String(parameter);
             string = PyString_AsString(stringval);
-            rc = sqlite3_bind_text(self->st, pos, string, -1, SQLITE_TRANSIENT);
+            buflen = PyString_Size(stringval);
+            rc = sqlite3_bind_text(self->st, pos, string, buflen, SQLITE_TRANSIENT);
             Py_DECREF(stringval);
             break;
         case TYPE_BUFFER:
@@ -185,9 +187,22 @@ int pysqlite_statement_bind_parameter(pysqlite_Statement* self, int pos, PyObjec
             break;
         case TYPE_UNKNOWN:
             rc = -1;
+            PyErr_Format(pysqlite_InterfaceError, "Parameter %d is of no supported type", pos);
     }
 
 final:
+    switch (rc) {
+        case SQLITE_TOOBIG:
+            PyErr_Format(pysqlite_DatabaseError, "Parameter %d is too big", pos);
+            break;
+        case SQLITE_RANGE:
+            PyErr_Format(pysqlite_DatabaseError, "Parameter index %d is out of range", pos);
+            break;
+        case SQLITE_NOMEM:
+            PyErr_Format(pysqlite_DatabaseError, "SQlite is out of memory for parameter %d", pos);
+            break;
+    }
+
     return rc;
 }
 
@@ -266,7 +281,7 @@ void pysqlite_statement_bind_parameters(pysqlite_Statement* self, PyObject* para
 
             if (rc != SQLITE_OK) {
                 if (!PyErr_Occurred()) {
-                    PyErr_Format(pysqlite_InterfaceError, "Error binding parameter %d - probably unsupported type.", i);
+                    PyErr_Format(pysqlite_InterfaceError, "Unknown error binding parameter %d.", i);
                 }
                 return;
             }
@@ -311,7 +326,7 @@ void pysqlite_statement_bind_parameters(pysqlite_Statement* self, PyObject* para
 
             if (rc != SQLITE_OK) {
                 if (!PyErr_Occurred()) {
-                    PyErr_Format(pysqlite_InterfaceError, "Error binding parameter :%s - probably unsupported type.", binding_name);
+                    PyErr_Format(pysqlite_InterfaceError, "Unknown error binding parameter %s.", binding_name);
                 }
                 return;
            }
@@ -421,7 +436,7 @@ void pysqlite_statement_dealloc(pysqlite_Statement* self)
         PyObject_ClearWeakRefs((PyObject*)self);
     }
 
-    self->ob_type->tp_free((PyObject*)self);
+    Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
 /*
@@ -495,8 +510,7 @@ static int pysqlite_check_remaining_sql(const char* tail)
 }
 
 PyTypeObject pysqlite_StatementType = {
-        PyObject_HEAD_INIT(NULL)
-        0,                                              /* ob_size */
+        PyVarObject_HEAD_INIT(NULL, 0)
         MODULE_NAME ".Statement",                       /* tp_name */
         sizeof(pysqlite_Statement),                     /* tp_basicsize */
         0,                                              /* tp_itemsize */
